from __future__ import absolute_import, division, print_function
from .store import Entry
import sys, warnings

if sys.version_info[0] < 3:
    str = unicode

EXTENSIONS = ['.yaml', '.yml']


def make_entry(key, dict):
    notes = ' | '.join(str(dict[key]) for key in ['L', 'N'] if key in dict)
    return Entry(key=key,
                 user=str(dict.get('U', '')),
                 password=str(dict.get('P', '')),
                 notes=notes)


def parse_entries(src):
    warnings.warn(
        "YAML support is deprecated and will be removed in the next version",
        DeprecationWarning)
    import yaml
    try:
        from yaml import CLoader as Loader
    except ImportError:
        from yaml import Loader as Loader
    root_node = yaml.load(src, Loader=Loader)
    return list(_collect_entries(root_node, ''))


def _collect_entries(current_node, current_key):
    # list of accounts?
    if isinstance(current_node, list):
        for child_node in current_node:
            assert isinstance(child_node, dict), "expected list of accounts"
            yield make_entry(current_key, child_node)
        return

    # single acccount?
    if isinstance(current_node, dict) and 'P' in current_node:
        yield make_entry(current_key, current_node)
        return

    # single password?
    if not isinstance(current_node, dict):
        yield Entry(key=current_key,
                    user='',
                    password=str(current_node),
                    notes='')
        return

    # otherwise: subtree!
    for key, child_node in current_node.items():
        # ignore entries in parentheses
        if key.startswith('('):
            continue

        # recurse
        for entry in _collect_entries(child_node, current_key + '.' + key if
                                      current_key else key):
            yield entry



from __future__ import absolute_import, division, print_function
import os, os.path, tempfile
import pytest
import pw._gpg
from pw._gpg import is_encrypted, has_armor, unencrypted_ext, decrypt, encrypt


@pytest.fixture(scope='module')
def dirname():
    return os.path.dirname(os.path.abspath(__file__))


def setup_module(module):
    # override GPG homedir
    pw._gpg._OVERRIDE_HOMEDIR = os.path.join(dirname(), 'keys')


def test_detection():
    # is_encrypted
    assert not is_encrypted('test.txt')
    assert is_encrypted('test.asc')
    assert is_encrypted('test.txt.asc')
    assert is_encrypted('test.gpg')
    assert is_encrypted('test.txt.gpg')

    # has_armor
    with pytest.raises(ValueError):
        has_armor('test.txt')
    assert has_armor('test.asc')
    assert has_armor('test.txt.asc')
    assert not has_armor('test.gpg')
    assert not has_armor('test.txt.gpg')

    # unencrypted_ext
    assert unencrypted_ext('test.txt') == '.txt'
    assert unencrypted_ext('test.asc') == ''
    assert unencrypted_ext('test.txt.asc') == '.txt'
    assert unencrypted_ext('test.gpg') == ''
    assert unencrypted_ext('test.txt.gpg') == '.txt'


@pytest.mark.parametrize("filename", ["db.pw.asc", "db.pw.gpg"])
def test_decrypt(dirname, filename):
    # manually decrypt password file & compare with unencrypted file in repository
    decrypted = decrypt(os.path.join(dirname, filename))
    unencrypted = open(os.path.join(dirname, 'db.pw'), 'rb').read()
    assert decrypted == unencrypted


@pytest.mark.parametrize("filename", ["db.pw.asc", "db.pw.gpg"])
def test_encrypt(dirname, filename):
    # load unencrypted password file
    unencrypted = open(os.path.join(dirname, 'db.pw'), 'rb').read()

    # encrypt into temporary file, decrypt again, and compare result
    fp = tempfile.NamedTemporaryFile(delete=False, suffix=filename)
    try:
        fp.close()

        encrypt(recipient='test.user@localhost',
                dest_path=fp.name,
                content=unencrypted)
        decrypted = decrypt(fp.name)
        assert decrypted == unencrypted
    finally:
        os.unlink(fp.name)



# coding: utf-8
from __future__ import absolute_import, division, print_function
from functools import partial
from click.testing import CliRunner
import os.path, sys, tempfile
import pytest
import pw, pw.__main__
import pyperclip


@pytest.fixture()
def dirname():
    return os.path.dirname(os.path.abspath(__file__))


def setup_module(module):
    # override GPG homedir
    pw._gpg._OVERRIDE_HOMEDIR = os.path.join(dirname(), 'keys')


@pytest.fixture(scope='module',
                params=['db.pw', 'db.pw.gpg', 'db.pw.asc', 'db.yaml'])
def runner(request):
    # instantiate runner and provide database path
    runner = CliRunner()
    abspath = os.path.join(dirname(), request.param)
    return lambda *args: runner.invoke(pw.__main__.pw, ('--file', abspath) + args)


def test_version(runner):
    result = runner("--version")
    assert result.exit_code == 0
    assert result.output.strip().startswith("pw version ")


@pytest.mark.parametrize("args, exit_code, output_expected", [
    # default query
    (
        [],
        0,
        u"""
goggles: alice@gogglemail.com
   https://mail.goggles.com/
   second line
goggles: bob+spam@gogglemail.com
laptop: alice | default user
laptop: bob
phones.myphone
phones.samson
router: ädmin | multiple (...)
        """,
    ),
    # querying for path and user
    (
        ["goggle"],
        0,
        """
goggles: alice@gogglemail.com
   https://mail.goggles.com/
   second line
goggles: bob+spam@gogglemail.com
        """,
    ),
    (
        ["bob@"],
        0,
        """
goggles: bob+spam@gogglemail.com
laptop: bob
        """,
    ),
    (
        ["bob@goggle"],
        0,
        "goggles: bob+spam@gogglemail.com",
    ),
    (
        ["goggle", "bob"],
        0,
        "goggles: bob+spam@gogglemail.com",
    ),
    (
        ["bob@goggle", "bob"],
        0,
        "",
    ),
    # strictness
    (
        ["--strict", "myphone"],
        0,
        "phones.myphone",
    ),
    (
        ["--strict", "phones"],
        2,
        "error: multiple or no records found (but using --strict flag)",
    ),
])  # yapf: disable
def test_query(runner, args, exit_code, output_expected):
    result = runner("--echo", "--user", *args)
    assert result.exit_code == exit_code
    assert result.output.strip() == output_expected.strip()


def test_missing():
    runner = CliRunner()
    result = runner.invoke(pw.__main__.pw, ('--file', 'XXX'))
    assert result.exit_code == 1
    assert "error: password store not found at 'XXX'" == result.output.strip()


CLIPBOARD_NOT_TOUCHED = u'CLIPBOARD_NOT_TOUCHED'

@pytest.mark.parametrize("args, output_expected, clipboard_expected", [
    (
        ["laptop", "bob"],
        "laptop: bob | *** PASSWORD COPIED TO CLIPBOARD ***",
        "b0b",
    ),
    (
        ["--copy", "laptop", "bob"],
        "laptop: bob | *** PASSWORD COPIED TO CLIPBOARD ***",
        "b0b",
    ),
    (
        ["--copy", "--user", "laptop", "bob"],
        "laptop: bob | *** USERNAME COPIED TO CLIPBOARD ***",
        "bob",
    ),
    (
        ["--echo", "laptop", "bob"],
        "laptop: bob | b0b",
        CLIPBOARD_NOT_TOUCHED,
    ),
    (
        ["--echo", "--user", "laptop", "bob"],
        "laptop: bob",
        CLIPBOARD_NOT_TOUCHED,
    ),
    (
        ["--raw", "laptop", "bob"],
        "b0b",
        CLIPBOARD_NOT_TOUCHED,
    ),
    (
        ["--raw", "--user", "laptop", "bob"],
        "bob",
        CLIPBOARD_NOT_TOUCHED,
    ),
])  # yapf: disable
def test_modes(runner, args, output_expected, clipboard_expected):
    pyperclip.copy(CLIPBOARD_NOT_TOUCHED)
    result = runner(*args)
    assert result.exit_code == 0
    assert result.output.strip() == output_expected.strip()
    assert pyperclip.paste() == clipboard_expected.strip()


@pytest.mark.parametrize("args, use_clipboard, length_expected", [
    (
        ["--gen"],
        True,
        pw.__main__.RANDOM_PASSWORD_DEFAULT_LENGTH,
    ),
    (
        ["--gen", "--copy"],
        True,
        pw.__main__.RANDOM_PASSWORD_DEFAULT_LENGTH,
    ),
    (
        ["--gen", "--echo"],
        False,
        pw.__main__.RANDOM_PASSWORD_DEFAULT_LENGTH,
    ),
    (
        ["--gen", "--echo", "--raw"],
        False,
        pw.__main__.RANDOM_PASSWORD_DEFAULT_LENGTH,
    ),
    (
        ["--gen", "8"],
        True,
        8,
    ),
    (
        ["--gen", "--copy", "8"],
        True,
        8,
    ),
    (
        ["--gen", "--echo", "8"],
        False,
        8,
    ),
    (
        ["--gen", "--echo", "--raw", "8"],
        False,
        8,
    ),
])  # yapf: disable
def test_gen(runner, args, use_clipboard, length_expected):
    pyperclip.copy(CLIPBOARD_NOT_TOUCHED)
    result = runner(*args)
    assert result.exit_code == 0
    output = result.output.strip()
    clipboard = pyperclip.paste()
    if use_clipboard:
        assert output == "*** PASSWORD COPIED TO CLIPBOARD ***"
        assert len(clipboard) == length_expected
        assert all(c in pw.__main__.RANDOM_PASSWORD_ALPHABET
                   for c in clipboard)
    else:
        assert len(output) == length_expected
        assert all(c in pw.__main__.RANDOM_PASSWORD_ALPHABET for c in output)
        assert clipboard == CLIPBOARD_NOT_TOUCHED


def test_edit_without_changes(runner):
    os.environ['PW_EDITOR'] = 'echo CALLED FOR'
    os.environ['PW_GPG_RECIPIENT'] = 'test.user@localhost'
    result = runner('--edit')
    assert result.exit_code == 0
    assert result.output.strip() == 'not modified'


def test_edit_editor_missing(runner):
    os.environ.pop('PW_EDITOR', None)
    os.environ['PW_GPG_RECIPIENT'] = 'test.user@localhost'
    result = runner("--edit")
    assert result.exit_code == 1
    assert result.output.strip().startswith('error: no editor set')


@pytest.fixture(scope='module', params=['db.pw.gpg', 'db.pw.asc'])
def encrypted_runner(request):
    return runner(request)


def test_edit_recipient_missing(encrypted_runner):
    os.environ['PW_EDITOR'] = 'echo CALLED FOR'
    os.environ.pop('PW_GPG_RECIPIENT', None)
    result = encrypted_runner("--edit")
    assert result.exit_code == 1
    assert result.output.strip().startswith('error: no recipient set')


def test_edit_file_missing():
    os.environ['PW_EDITOR'] = 'echo CALLED FOR'
    os.environ['PW_GPG_RECIPIENT'] = 'test.user@localhost'
    runner = CliRunner()
    result = runner.invoke(pw.__main__.pw, ('--file', 'XXX', '--edit'))
    assert result.exit_code == 1
    assert "error: password store not found at 'XXX'" == result.output.strip()


@pytest.mark.parametrize('filename, addendum', [
    ('db.pw', 'fancy_new_entry: user pass interesting notes'),
    ('db.pw.gpg', 'fancy_new_entry: user pass interesting notes'),
    ('db.pw.asc', 'fancy_new_entry: user pass interesting notes'),
    ('db.yaml', 'fancy_new_entry: {U: user, P: pass, N: interesting notes}'),
])
def test_edit_with_changes(dirname, filename, addendum):
    fp = tempfile.NamedTemporaryFile(delete=False, suffix=filename)
    try:
        # copy password file to temporary file
        abspath = os.path.join(dirname, filename)
        original = open(abspath, 'rb').read()
        fp.write(original)
        fp.close()

        # call pw --edit and modify file
        editor = os.path.join(dirname, 'add_a_line.py')
        os.environ['PW_EDITOR'] = '%s "%s" "%s"' % (sys.executable, editor,
                                                    addendum)
        os.environ['PW_GPG_RECIPIENT'] = 'test.user@localhost'
        runner = CliRunner()
        result = runner.invoke(pw.__main__.pw, ('--file', fp.name, '--edit'))
        assert result.exit_code == 0
        assert result.output.strip() == ''

        # try to find new entry
        store = pw.Store.load(fp.name)
        result = store.search(key_pattern='fancy', user_pattern='')
        assert len(result) == 1
        assert result[0] == pw.Entry('fancy_new_entry', 'user', 'pass',
                                     'interesting notes')
    finally:
        os.unlink(fp.name)



# coding: utf-8
from __future__ import absolute_import, division, print_function
import pytest
import os.path
import pw
from pw.store import _normalized_key, _parse_entries, Entry, Store, SyntaxError


def dirname():
    return os.path.dirname(os.path.abspath(__file__))


def setup_module(module):
    # override GPG homedir
    pw._gpg._OVERRIDE_HOMEDIR = os.path.join(dirname(), 'keys')


def test_normalized_key():
    assert _normalized_key('My secret aCcOuNt') == 'my_secret_account'


@pytest.mark.parametrize("src, expected", [
    (
        u"",
        [],
    ),
    (
        u"""
# this is a comment
        """,
        [],
    ),
    (
        u"""
key pass
key: pass
#
"key word" pass
"key word": pass
#
key "pass word"
key: "pass word"
#
"key word" "pass word"
"key word": "pass word"
        """,
        [
            Entry(key='key', user='', password='pass', notes=''),
            Entry(key='key', user='', password='pass', notes=''),
            Entry(key='key word', user='', password='pass', notes=''),
            Entry(key='key word', user='', password='pass', notes=''),
            Entry(key='key', user='', password='pass word', notes=''),
            Entry(key='key', user='', password='pass word', notes=''),
            Entry(key='key word', user='', password='pass word', notes=''),
            Entry(key='key word', user='', password='pass word', notes=''),
        ]
    ),
    (
        u"""
key user pass
key: user pass
#
"key word" "user name" "pass word"
"key word": "user name" "pass word"
        """,
        [
            Entry(key='key', user='user', password='pass', notes=''),
            Entry(key='key', user='user', password='pass', notes=''),
            Entry(key='key word', user='user name', password='pass word', notes=''),
            Entry(key='key word', user='user name', password='pass word', notes=''),
        ],
    ),
    (
        u"""
key user pass these are some interesting notes
key: user pass these are some interesting notes
#
"key word" "user name" "pass word" these are some interesting notes
"key word": "user name" "pass word" these are some interesting notes
        """,
        [
            Entry(key='key', user='user', password='pass', notes='these are some interesting notes'),
            Entry(key='key', user='user', password='pass', notes='these are some interesting notes'),
            Entry(key='key word', user='user name', password='pass word', notes='these are some interesting notes'),
            Entry(key='key word', user='user name', password='pass word', notes='these are some interesting notes'),
        ],
    ),
    (
        u"""
key pass
    notes line 1
    notes line 2
key: pass
    notes line 1
    notes line 2
#
key "" pass notes line 0
    notes line 1
    notes line 2
key: "" pass notes line 0
    notes line 1
    notes line 2
#
key "" pass notes line 0
    # notes line 0.5
    notes line 1
    # notes line 1.5
    notes line 2
key "" pass notes line 0
    # notes line 0.5
    notes line 1
    # notes line 1.5
    notes line 2
        """,
        [
            Entry(key='key', user='', password='pass', notes='notes line 1\nnotes line 2'),
            Entry(key='key', user='', password='pass', notes='notes line 1\nnotes line 2'),
            Entry(key='key', user='', password='pass', notes='notes line 0\nnotes line 1\nnotes line 2'),
            Entry(key='key', user='', password='pass', notes='notes line 0\nnotes line 1\nnotes line 2'),
            Entry(key='key', user='', password='pass', notes='notes line 0\n# notes line 0.5\nnotes line 1\n# notes line 1.5\nnotes line 2'),
            Entry(key='key', user='', password='pass', notes='notes line 0\n# notes line 0.5\nnotes line 1\n# notes line 1.5\nnotes line 2'),
        ],
    ),
    (
        u"""
key user pass
    notes line 1
    notes line 2
key: user pass
    notes line 1
    notes line 2
#
key user pass notes line 0
    notes line 1
    notes line 2
key: user pass notes line 0
    notes line 1
    notes line 2
#
key user pass notes line 0
    # notes line 0.5
    notes line 1
    # notes line 1.5
    notes line 2
key user pass notes line 0
    # notes line 0.5
    notes line 1
    # notes line 1.5
    notes line 2
        """,
        [
            Entry(key='key', user='user', password='pass', notes='notes line 1\nnotes line 2'),
            Entry(key='key', user='user', password='pass', notes='notes line 1\nnotes line 2'),
            Entry(key='key', user='user', password='pass', notes='notes line 0\nnotes line 1\nnotes line 2'),
            Entry(key='key', user='user', password='pass', notes='notes line 0\nnotes line 1\nnotes line 2'),
            Entry(key='key', user='user', password='pass', notes='notes line 0\n# notes line 0.5\nnotes line 1\n# notes line 1.5\nnotes line 2'),
            Entry(key='key', user='user', password='pass', notes='notes line 0\n# notes line 0.5\nnotes line 1\n# notes line 1.5\nnotes line 2'),
        ],
    ),
    (
        u"""
"السلام عليكم": 임요환 Нет Εις το επανιδείν
        """,
        [
            Entry(key=u'السلام عليكم', user=u'임요환', password=u'Нет', notes=u'Εις το επανιδείν'),
        ],
    ),
])  # yapf: disable
def test_parse_entries(src, expected):
    entries = _parse_entries(src.strip())
    assert entries == expected

@pytest.mark.parametrize("src, expected_error_prefix", [
    (
        u"""
foo: bar
  note line 1

  note line 2
        """,
        "line 4: expecting entry (",
    ),
    (
        u"""
foo: bar
  note line 1
# non-indented comment
  note line 2
        """,
        "line 4: expecting entry (",
    ),
    (
        u"""
foo: bar
baz
boink: zonk
        """,
        "line 2: expecting entry or notes (",
    ),
    (
        u"""
foo": bar
        """,
        "line 1: No closing quotation (",
    ),
    (
        u"""
foo: "bar
        """,
        "line 1: No closing quotation (",
    ),
    (
        u"""
foo: bar "baz
        """,
        "line 1: No closing quotation (",
    ),
])  # yapf: disable
def test_parse_entries_syntax_errors(src, expected_error_prefix):
    with pytest.raises(SyntaxError) as excinfo:
        _parse_entries(src.strip())
    assert str(excinfo.value).startswith(expected_error_prefix)


@pytest.fixture(scope='module',
                params=['db.pw', 'db.pw.gpg', 'db.pw.asc', 'db.yaml'])
def store(request):
    abspath = os.path.join(dirname(), request.param)
    return Store.load(abspath)


def test_store_entries(store):
    expected = [
        Entry('laptop', 'alice', '4l1c3', 'default user'),
        Entry('laptop', 'bob', 'b0b', ''),
        Entry('goggles', 'alice@gogglemail.com', '12345', 'https://mail.goggles.com/\nsecond line'),
        Entry('goggles', 'bob+spam@gogglemail.com', 'abcde', ''),
        Entry('router', u'ädmin', 'gamma zeta', 'multiple\nlines\nof\nnotes'),
        Entry('phones.myphone', '', '0000', ''),
        Entry('phones.samson', '', '111', ''),
    ]  # yapf: disable
    expected = sorted(expected, key=lambda e: e.key)
    got = sorted(store.entries, key=lambda e: e.key)
    assert got == expected


def test_store_search(store):
    # search for key
    got = store.search(key_pattern='oggle', user_pattern='')
    expected = [
        Entry('goggles', 'alice@gogglemail.com', '12345',
              'https://mail.goggles.com/\nsecond line'),
        Entry('goggles', 'bob+spam@gogglemail.com', 'abcde', ''),
    ]
    assert got == expected

    # search for user
    got = store.search(key_pattern='', user_pattern='bob')
    expected = [
        Entry('goggles', 'bob+spam@gogglemail.com', 'abcde', ''),
        Entry('laptop', 'bob', 'b0b', ''),
    ]
    assert got == expected

    # search for both
    got = store.search(key_pattern='oggle', user_pattern='spam')
    expected = [Entry('goggles', 'bob+spam@gogglemail.com', 'abcde', ''), ]
    assert got == expected



from __future__ import absolute_import, division, print_function
import os.path, sys
from collections import namedtuple
from io import StringIO
if sys.version_info[0] < 3:
    from ushlex import shlex
else:
    from shlex import shlex
from . import _gpg

Entry = namedtuple('Entry', ['key', 'user', 'password', 'notes'])


def _normalized_key(key):
    return key.replace(' ', '_').lower()


class Store:
    """Password store."""

    def __init__(self, path, entries):
        # normalize keys
        self.entries = [e._replace(key=_normalized_key(e.key))
                        for e in entries]
        self.path = path

    def search(self, key_pattern, user_pattern):
        """Search database for given key and user pattern."""
        # normalize key
        key_pattern = _normalized_key(key_pattern)

        # search
        results = []
        for entry in self.entries:
            if key_pattern in entry.key and user_pattern in entry.user:
                results.append(entry)

        # sort results according to key (stability of sorted() ensures that the order of accounts for any given key remains untouched)
        return sorted(results, key=lambda e: e.key)

    @staticmethod
    def load(path):
        """Load password store from file."""
        # load source (decrypting if necessary)
        if _gpg.is_encrypted(path):
            src_bytes = _gpg.decrypt(path)
        else:
            src_bytes = open(path, 'rb').read()
        src = src_bytes.decode('utf-8')

        # parse database source
        ext = _gpg.unencrypted_ext(path)
        if ext in ['.yml', '.yaml']:
            from . import _yaml
            entries = _yaml.parse_entries(src)
        else:
            entries = _parse_entries(src)

        return Store(path, entries)


class SyntaxError(Exception):
    def __init__(self, lineno, line, reason):
        super(SyntaxError, self).__init__('line %s: %s (%r)' %
                                          (lineno + 1, reason, line))


_EXPECT_ENTRY = 'expecting entry'
_EXPECT_ENTRY_OR_NOTES = 'expecting entry or notes'


def _parse_entries(src):
    entries = []
    state = _EXPECT_ENTRY

    for lineno, line in enumerate(src.splitlines()):
        # empty lines are skipped (but also terminate the notes section)
        sline = line.strip()
        if not sline or line.startswith('#'):
            state = _EXPECT_ENTRY
            continue

        # non-empty line with leading spaces is interpreted as a notes line
        if line[0] in [' ', '\t']:
            if state != _EXPECT_ENTRY_OR_NOTES:
                raise SyntaxError(lineno, line, state)

            # add line of notes
            notes = entries[-1].notes
            if notes:
                notes += "\n"
            notes += sline
            entries[-1] = entries[-1]._replace(notes=notes)
            continue

        # otherwise, parse as an entry
        sio = StringIO(line)
        lexer = shlex(sio, posix=True)
        lexer.whitespace_split = True

        try:
            key = lexer.get_token()
        except ValueError as e:
            raise SyntaxError(lineno, line, str(e))
        key = key.rstrip(':')
        assert key

        try:
            user = lexer.get_token()
        except ValueError as e:
            raise SyntaxError(lineno, line, str(e))

        try:
            password = lexer.get_token()
        except ValueError as e:
            raise SyntaxError(lineno, line, str(e))

        if not user and not password:
            raise SyntaxError(lineno, line, state)

        if not password:
            password = user
            user = notes = u''
        else:
            password = password
            notes = sio.read().strip()

        entries.append(Entry(key, user, password, notes))
        state = _EXPECT_ENTRY_OR_NOTES

    return entries



from __future__ import absolute_import, division, print_function
from setuptools import setup
import ast, io, re
from os.path import join, dirname, abspath

# determine __version__ from pw.py source (adapted from mitsuhiko)
VERSION_RE = re.compile(r'__version__\s+=\s+(.*)')

with io.open('pw/__init__.py', encoding='utf-8') as fp:
    version_code = VERSION_RE.search(fp.read()).group(1)
    version = str(ast.literal_eval(version_code))

# read long description and convert to RST
long_description = io.open(
    join(
        dirname(abspath(__file__)), 'README.md'),
    encoding='utf-8').read()
try:
    import pypandoc
    long_description = pypandoc.convert(long_description, 'rst', format='md')
except ImportError:
    pass

# package metadata
setup(
    name='pw',
    version=version,
    description='Search in GPG-encrypted password file.',
    author='Michael Walter',
    author_email='michael.walter@gmail.com',
    url='https://github.com/catch22/pw',
    license='MIT',
    packages=['pw'],
    entry_points={
        'console_scripts': ['pw = pw.__main__:pw']
    },
    classifiers=[
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'License :: OSI Approved :: MIT License',
    ],
    long_description=long_description,
    install_requires=['click>=5.1', 'colorama', 'pyperclip>=1.5.11', 'ushlex'],
    tests_require=['pytest', 'PyYAML'])



from __future__ import absolute_import, division, print_function
import os.path
import subprocess

_HAS_ARMOR = {'.gpg': False, '.asc': True}
_EXTENSIONS = _HAS_ARMOR.keys()
_OVERRIDE_HOMEDIR = None  # useful for unit tests


def is_encrypted(path):
    _, ext = os.path.splitext(path)
    return ext in _EXTENSIONS


def has_armor(path):
    _, ext = os.path.splitext(path)
    if ext not in _EXTENSIONS:
        raise ValueError('File extension not recognized as encrypted (%r).' %
                         ext)
    return _HAS_ARMOR[ext]


def unencrypted_ext(path):
    root, ext = os.path.splitext(path)
    if ext in _EXTENSIONS:
        _, ext = os.path.splitext(root)
    return ext


def _base_args():
    args = ['gpg2', '--use-agent', '--quiet', '--batch', '--yes']
    if _OVERRIDE_HOMEDIR:
        args += ['--homedir', _OVERRIDE_HOMEDIR]
    return args


def decrypt(path):
    args = ['--decrypt', path]
    return subprocess.check_output(_base_args() + args)


def encrypt(recipient, dest_path, content):
    args = ["--encrypt"]
    if has_armor(dest_path):
        args += ["--armor"]
    args += ["--recipient", recipient, "--output", dest_path]
    popen = subprocess.Popen(_base_args() + args,
                             stdin=subprocess.PIPE,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
    stdout, stderr = popen.communicate(content)
    assert popen.returncode == 0, stderr
